'''
This is a simple tool to convert the listing produced by the
A09 assembler into Epson HX-20 BASIC statements.
The resulting BASIC program loads the machine code into memory.
The code can then be executed by an EXEC statement.

In the DATA statement starting addresses for a range of opcodes
or data are identifyable by a length of four characters. 
All opcodes or data bytes are two characters long.
'''

import sys

# ------------------------------------------
def go(s):
	'''
   For Epson HX-20.
	Convert 6301 assembler listing file "s" into BASIC.
   '''
		
	fIn = open(s);
	ss = fIn.readlines();
	fIn.close();
	
	nLines = len(ss);

	# these values have to be adapted
	# where the PRINT "Done." is placed
	nStop  = 120
	# where the HEX->DEC subroutines start
	nHex = nStop + 50
	nHex4 = nHex + 40
	nHex2 = nHex + 70
	
	n = 10
	print str(n)+' REM --- Epson HX-20      ---'
	n = n+10
	print str(n)+' REM --- Hex Code Loader  ---'
	n = n+10
	print str(n)+' REM --- M. Hepperle 2018 ---'
	n = n+10

	# skip MEMSET line 
	nMemSet = n
	n = n+10
	print str(n)+' N%=0'
	n = n+10
	nLoop = n
	print str(n)+' READ C$'
	n = n+10
	print str(n)+' IF C$="DONE" THEN '+str(nStop)
	n = n+10
	print str(n)+' N%=N%+1 : IF N%=8 THEN PRINT "."; : N%=0'
	n = n+10
	print str(n)+' C%=0 : I%=1'
	n = n+10
	# new address, DATA MUST start with an address!
	print str(n)+' IF LEN(C$)=4 THEN GOSUB ' + str(nHex4) + ' : A% = C% : GOTO '+str(nLoop)
	n = n+10
	# new opcode
	print str(n)+' GOSUB ' + str(nHex2) + ' : POKE A%,C% : A%=A%+1 : GOTO '+str(nLoop)
	n = n+10

	if n>nStop:
		print '*** ERROR: increase nStop to at least '+str(n)

	n = nStop
	print str(n)+' PRINT "Done."'
	n = n+10
	print str(n)+' REM --- call the function'
	n = n+10
	print str(n)+' DEFUSR1=&H0A40'
	n = n+10
	print str(n)+' PRINT USR1(CHR$(0)+CHR$(32)+CHR$(0)+CHR$(64)+"Hello World")'
	n = n+10
	print str(n)+' REM or (if no parameters):'
	n = n+10
	print str(n)+' REM EXEC &H0A40'
	n = n+10
	print str(n)+' END'
	n = n+10

	nHexDigit = n+80
	print str(n) + ' REM C$(HEX) -> C%(DEC), set C%=0 and I%=1 before GOSUB'
	n = n+10
	print str(n) + ' REM Entry HEX4'
	n = n+10
	print str(n) + ' X$=MID$(C$,I%,1) : GOSUB '+str(nHexDigit) + ' : C%=C%+4096*X% : I%=I%+1'
	n = n+10
	print str(n) + ' X$=MID$(C$,I%,1) : GOSUB '+str(nHexDigit) + ' : C%=C%+256*X% : I%=I%+1'
	n = n+10
	print str(n) + ' REM Entry HEX2'
	n = n+10
	print str(n) + ' X$=MID$(C$,I%,1) : GOSUB '+str(nHexDigit) + ' : C%=C%+16*X% : I%=I%+1'
	n = n+10
	print str(n) + ' X$=MID$(C$,I%,1) : GOSUB '+str(nHexDigit) + ' : C%=C%+X%'
	n = n+10
	print str(n) + ' RETURN'
	n = n+10
	# nHexDigit = n
	print str(n) + ' X%=ASC(X$) : IF X%>64 THEN X%=X%-55 ELSE X%=X%-48'
	n = n+10
	print str(n) + ' RETURN'
	n = n+10

	line=0;
	address = 0

	startAddress = 65536
	endAddress = 0

	sLine = ''

	while line < nLines:
		l = ss[line].replace("\n","")

		# this is the End
		if l.startswith('SYMBOL TABLE'):
			break

		# continuation line has no blank in the first column
		if l[0:1] != ' ':
		#{
			# skip
			line = line+1
			continue
		#}

		addr = l[1:5].strip()
		
		if len(addr) == 4:
		#{
			try:
			#{
				addrDec = int(addr,16)

				if addrDec < startAddress:
				#{
					startAddress = addrDec
				#}

				if addrDec > endAddress:
				#{
					endAddress = addrDec
				#}

				if addrDec != address:
				#{
					# a step in addresses - 
					# output new start address
					address = addrDec
					sLine = sLine + addr + ','
				#}

				opcodes = l[6:20].strip()
				i = 0

				while i < len(opcodes):
				#{
					sLine = sLine + opcodes[i:i+2] + ','
					i = i+2

					# update high water mark
					if address > endAddress:
					#{
						endAddress = address
					#}

					# next free address or start of BASIC for MEMSET
					address = address+1

					if len(sLine)>57:
					#{
						print str(n) + ' DATA ' + sLine[0:len(sLine)-1]
						sLine = ''
						n = n+1
					#}
				#}
			#}
			except:
			#{
				addrDec = 0
			#}
		#}

		line = line+1
		if line > 50000:
		#{
			break
		#}

	sLine = sLine + 'DONE,'
	if len(sLine)>0:
		print str(n) + ' DATA ' + sLine[0:len(sLine)-1]
		sLine = ''
		n = n+10

	# insert MEMSET line above
	print str(nMemSet) + ' MEMSET &H' + hex(endAddress+1).upper()[2:]

	# terminate transfer with ^Z
	print '\032'

	if 1==1:
		print 'The binary code resides between &H'+hex(startAddress).upper()[2:]+' and &H' + hex(endAddress).upper()[2:]+'.'
		print 'Thus we need to shift the start of the BASIC'
		print 'program and data area to &H' + hex(endAddress+1).upper()[2:] + '.'
		print 'The assembler code should end with an RTS instruction.'
		print 'If the code requires no parameters, you can execute it with'
		print 'EXEC &H' + hex(startAddress).upper()[2:]
		print 'If it takes a parameter, wrap it into a USR function.'

# ------------------------------------------

if __name__ == "__main__":
   if len(sys.argv)>1:
      basePath = "D:\\HP\\Epson HX-20\\ASM\\"
      basePath = './'
      fileName = sys.argv[1]
      go(basePath + fileName)
   else:
      print 'Usage: LST2BAS listing.lst'

